@@ -16,7 +16,7 @@ import matplotlib.patches as patches
 import matplotlib.pyplot as plt
 import numpy as np
 import torch
-from scipy import fftpack, ndimage, optimize, spatial
+from scipy import fftpack, ndimage, optimize, spatial, stats
 from skimage import exposure
 from skimage.util import random_noise
 from sklearn.feature_extraction.image import extract_patches_2d
@@ -277,12 +277,12 @@ def img_resize(image_data, rs, round_=False):
 def cv_resize(img, rs, round_=False):
     """
     Wrapper for open-cv resize function
-    
+
     Args:
         img (2D numpy array): input 2D image
         rs (tuple): target height and width
         round_(bool): rounding (in case of labeled pixels)
-    
+
     Returns:
         Resized image
     """
@@ -1103,7 +1103,7 @@ def extract_patches(images, masks, patch_size, num_patches, **kwargs):
     images_aug = np.concatenate(images_aug, axis=0)
     masks_aug = np.concatenate(masks_aug, axis=0)
     return images_aug, masks_aug
-        
+
 
 class datatransform:
     """
@@ -1139,6 +1139,9 @@ class datatransform:
         **contrast (bool or list ot tuple):
             Contrast level. You can pass min and max values as a list/tuple
             (Default [min, max] range: [5, 20])
+        **background (bool):
+            Adds/substracts asymmetric 2D gaussian of random width and intensity
+            from the image
         **resize (tuple):
             Values for image resizing
             [downscale factor (default: 2), upscale factor (default:1.5)]
@@ -1156,26 +1159,30 @@ class datatransform:
         self.dim_order_out = dim_order_out
         self.squeeze = squeeze_channels
         self.rotation = kwargs.get('rotation')
+        self.background = kwargs.get('background')
         self.gauss = kwargs.get('gauss')
-        if isinstance(self.gauss, bool):
+        if self.gauss is True:
             self.gauss = [0, 50]
+        self.jitter = kwargs.get("jitter")
+        if self.jitter is True:
+            self.jitter = [0, 50]
         self.poisson = kwargs.get('poisson')
-        if isinstance(self.poisson, bool):
+        if self.poisson is True:
             self.poisson = [30, 40]
         self.salt_and_pepper = kwargs.get('salt_and_pepper')
-        if isinstance(self.salt_and_pepper, bool):
+        if self.salt_and_pepper is True:
             self.salt_and_pepper = [0, 50]
         self.blur = kwargs.get('blur')
-        if isinstance(self.blur, bool):
+        if self.blur is True:
             self.blur = [1, 50]
         self.contrast = kwargs.get('contrast')
-        if isinstance(self.contrast, bool):
+        if self.contrast is True:
             self.contrast = [5, 20]
         self.zoom = kwargs.get('zoom')
-        if isinstance(self.zoom, bool):
+        if self.zoom is True:
             self.zoom = 2  # [min, max] zoom
         self.resize = kwargs.get('resize')
-        if isinstance(self.resize, bool):
+        if self.resize is True:
             self.resize = [2, 1.5]
         if seed is not None:
             np.random.seed(seed)
@@ -1193,6 +1200,19 @@ class datatransform:
             X_batch_noisy[i] = img_
         return X_batch_noisy, y_batch
 
+    def apply_jitter(self, X_batch, y_batch):
+        """
+        Random application of jitter noise to each training image in a stack
+        """
+        n, h, w = X_batch.shape[0:3]
+        X_batch_noisy = np.zeros((n, h, w))
+        for i, img in enumerate(X_batch):
+            jitter_amount = np.random.randint(self.jitter[0], self.jitter[1]) / 10
+            shift_arr = stats.poisson.rvs(jitter_amount, loc=0, size=h)
+            img_ = np.array([np.roll(row, z) for row, z in zip(img, shift_arr)])
+            X_batch_noisy[i] = img_
+        return X_batch_noisy, y_batch
+
     def apply_poisson(self, X_batch, y_batch):
         """
         Random application of poisson noise to each training inage in a stack
@@ -1269,6 +1289,7 @@ class datatransform:
                 img, (shortdim, shortdim), interpolation=cv2.INTER_CUBIC)
             gt = cv2.resize(
                 gt, (shortdim, shortdim), interpolation=cv2.INTER_CUBIC)
+            img = np.clip(img, 0, 1)
             gt = np.around(gt)
             if len(gt.shape) != 3:
                 gt = np.expand_dims(gt, axis=2)
@@ -1276,6 +1297,26 @@ class datatransform:
             y_batch_z[i] = gt
         return X_batch_z, y_batch_z
 
+    def apply_background(self, X_batch, y_batch):
+        """
+        Emulates thickness variation in STEM or height variation in STM
+        """
+        def gauss2d(xy, x0, y0, a, b, fwhm):
+            return np.exp(-np.log(2)*(a*(xy[0]-x0)**2 + b*(xy[1]-y0)**2) / fwhm**2)
+        n, h, w = X_batch.shape[0:3]
+        X_batch_b = np.zeros((n, h, w))
+        x, y = np.meshgrid(
+            np.linspace(0, h, h), np.linspace(0, w, w), indexing='ij')
+        for i, img in enumerate(X_batch):
+            x0 = np.random.randint(0, h - h // 4)
+            y0 = np.random.randint(0, w - w // 4)
+            a, b = np.random.randint(10, 20, 2) / 10
+            fwhm = np.random.randint(min([h, w]) // 4, min([h, w]) - min([h, w]) // 2)
+            Z = gauss2d([x, y], x0, y0, a, b, fwhm)
+            img = img + 0.05 * np.random.randint(-10, 10) * Z
+            X_batch_b[i] = img
+        return X_batch_b, y_batch
+
     def apply_rotation(self, X_batch, y_batch):
         """
         Flips and rotates training images and correponding ground truth images
@@ -1343,20 +1384,24 @@ class datatransform:
         images = (images - images.min()) / images.ptp()
         if self.rotation:
             images, masks = self.apply_rotation(images, masks)
-        if self.zoom is not None:
+        if isinstance(self.zoom, int):
             images, masks = self.apply_zoom(images, masks)
-        if self.resize is not None:
+        if isinstance(self.resize, list) or isinstance(self.resize, tuple):
             images, masks = self.apply_imresize(images, masks)
-        if self.gauss is not None:
+        if isinstance(self.gauss, list) or isinstance(self.gauss, tuple):
             images, masks = self.apply_gauss(images, masks)
-        if self.poisson is not None:
+        if isinstance(self.jitter, list) or isinstance(self.jitter, tuple):
+            images, masks = self.apply_jitter(images, masks)
+        if isinstance(self.poisson, list) or isinstance(self.poisson, tuple):
             images, masks = self.apply_poisson(images, masks)
-        if self.salt_and_pepper is not None:
+        if isinstance(self.salt_and_pepper, list) or isinstance(self.salt_and_pepper, tuple):
             images, masks = self.apply_sp(images, masks)
-        if self.blur is not None:
+        if isinstance(self.blur, list) or isinstance(self.blur, tuple):
             images, masks = self.apply_blur(images, masks)
-        if self.contrast is not None:
+        if isinstance(self.contrast, list) or isinstance(self.contrast, tuple):
             images, masks = self.apply_contrast(images, masks)
+        if self.background:
+            images, masks = self.apply_background(images, masks)
         if self.squeeze:
             images, masks = self.squeeze_data(images, masks)
         if self.dim_order_out == 'channel_first':
